@@ -1,8 +1,8 @@
 /*
  * buffers.c - buffer management for the Linux file system degragmenter.
- * $Id: buffers.c,v 1.2 1994/05/22 22:59:26 linux Exp $
+ * $Id: buffers.c,v 1.5 1997/08/17 14:32:47 linux Exp $
  *
- * Copyright (C) 1992, 1993 Stephen Tweedie (sct@dcs.ed.ac.uk)
+ * Copyright (C) 1992, 1993, 1997 Stephen Tweedie (sct@dcs.ed.ac.uk)
  * 
  * This file may be redistributed under the terms of the GNU General
  * Public License.
@@ -320,6 +320,8 @@ void check_zone_nr (Block nr)
  */
 void read_current_block (Block nnr, char * addr)
 {
+	loff_t offset;
+	
 	if (debug)
 		printf ("DEBUG: read_block (&%ld, %d)\n", 
 			(long) nnr, (int) addr);
@@ -327,7 +329,9 @@ void read_current_block (Block nnr, char * addr)
 		return;
 	check_zone_nr(nnr);
 
-	if (block_size * (nnr) != nlseek (IN, block_size * (nnr), SEEK_SET))
+	offset = block_size * nnr;
+	
+	if (offset != nlseek (IN, offset, SEEK_SET))
 	{
 		io_error ("seek failed in read_block");
 		return;
@@ -349,12 +353,16 @@ void read_current_block (Block nnr, char * addr)
  */
 void write_current_block (Block nnr, char * addr)
 {
+	loff_t offset;
+
 	if (debug)
 		printf ("DEBUG: write_block(%ld, %d)\n", 
 			(long) nnr, (int) addr);
 	check_zone_nr(nnr);
 
-	if (block_size * (nnr) != nlseek (IN, block_size * (nnr), SEEK_SET))
+	offset = block_size * nnr;
+	
+	if (offset != nlseek (IN, offset, SEEK_SET))
 	{
 		io_error ("seek failed in write_block");
 		return;
@@ -711,5 +719,3 @@ void remap_disk_blocks (void)
 	assert (!count_rescue_buffers);
 	assert (free_buffers == pool_size);
 }
-
-       
