@@ -12,7 +12,7 @@
  *   map to disk.
  *
  * Copyleft  (C) 1993 Alexey Vovenko (vovenko@ixwin.ihep.su)
- * Copyright (C) 1992, 1993 Stephen Tweedie (sct@dcs.ed.ac.uk)
+ * Copyright (C) 1992, 1993, 1997 Stephen Tweedie (sct@dcs.ed.ac.uk)
  * Copyright (C) 1992 Remy Card (card@masi.ibp.fr)
  * Copyright (C) 1991 Linus Torvalds (torvalds@kruuna.helsinki.fi)
  * 
@@ -63,13 +63,13 @@ static void read_groups(void) {
            /* die here, if any bitmap alignment problems exist */
            /* It's quite improbable, that bitmaps are not byte aligned */
    if ((Super.s_blocks_per_group % 8) != 0) {
-      sprintf(s,"Strange blocks_per_group (%lu) value",
-              Super.s_blocks_per_group);
+      sprintf(s, "Strange blocks_per_group (%lu) value",
+              (unsigned long) Super.s_blocks_per_group);
       fatal_error(s);
    }         
    if ((Super.s_inodes_per_group % 8) != 0) {
-      sprintf(s,"Strange inodes_per_group (%lu) value",
-              Super.s_blocks_per_group);
+      sprintf(s, "Strange inodes_per_group (%lu) value",
+              (unsigned long) Super.s_blocks_per_group);
       fatal_error(s);
    }
 }
@@ -78,6 +78,8 @@ static void read_groups(void) {
  * for the remaining disk map tables (inodes, inodes & block bitmaps) */
 void read_tables (void)
 {
+    loff_t end_of_device;
+	
     if (debug)
 	printf ("DEBUG: read_tables()\n");
     if (BLOCK_SIZE != nlseek (IN, BLOCK_SIZE, SEEK_SET))
@@ -85,15 +87,27 @@ void read_tables (void)
     if (BLOCK_SIZE != nread (IN, super_block_buffer, BLOCK_SIZE))
 	die ("unable to read super-block");
     if (MAGIC != EXT2_SUPER_MAGIC)
-	die ("bad magic number in super-block");
+  	die ("bad magic number in super-block");
     if (ZONESIZE != 0 || BLOCK_SIZE != 1024)
-	die("Only 1k blocks/zones supported");
-    if (Super.s_state & EXT2_VALID_FS == 0)
-	die("Please run fsck first, filesystem is not marked valid\n");
+	die ("Only 1k blocks/zones supported");
+
+    /* 
+     * This modification is lousy, but better than the bug before  :-)  
+     */
+    if (!(Super.s_state & EXT2_VALID_FS) && !readonly)
+    	die ("Please run fsck first, filesystem is not marked valid\n");
     groups = (Super.s_blocks_count - Super.s_first_data_block + 
 	      Super.s_blocks_per_group - 1) /
-	Super.s_blocks_per_group;
+	      Super.s_blocks_per_group;
 
+    /* 
+     * Sanity check that llseek() is working OK if this is a large
+     * (>2GB) partition.  
+     */
+    end_of_device = ((loff_t) ZONES) * BLOCK_SIZE - BLOCK_SIZE;
+    if (llseek(IN, end_of_device, SEEK_SET) != end_of_device)
+	    die ("Error seeking to end of filesystem");
+	
     read_groups();
 
     inode_average_map = malloc (INODES * sizeof(*inode_average_map));
@@ -142,30 +156,36 @@ void show_super_stats(void) {
     char s[256];
     if (voyer_mode)
     {
-	sprintf (s, "%6ld block%s, %6ld free (%ld%%)", 
-		 ZONES,  (ZONES  != 1) ? "s" : "",FREEBLOCKSCOUNT,
-		 (100*FREEBLOCKSCOUNT) / ZONES);
+	sprintf (s, "%6lu block%s, %6lu free (%ld%%)", 
+		 (unsigned long) ZONES, (ZONES  != 1) ? "s" : "", 
+		 (unsigned long) FREEBLOCKSCOUNT,
+		 (100UL * FREEBLOCKSCOUNT) / ZONES);
 	add_comment(s);           
-	sprintf (s,"%6ld inode%s, %6ld free (%ld%%)", 
-		 INODES, (INODES != 1) ? "s" : "",FREEINODESCOUNT,
-		 (100*FREEINODESCOUNT) / INODES);
+	sprintf (s, "%6lu inode%s, %6lu free (%ld%%)", 
+		 (unsigned long) INODES, (INODES != 1) ? "s" : "", 
+		 (unsigned long) FREEINODESCOUNT,
+		 (100UL * FREEINODESCOUNT) / INODES);
 	add_comment(s);         
-	sprintf (s,"%d group%s,", 
+	sprintf (s, "%6u group%s,", 
 		 groups, (groups != 1) ? "s" : "");
 	add_comment(s);
 	display_comments("");           
     }
     else if (show)
     {
-	printf ("%ld inode%s\n", INODES, (INODES != 1) ? "s" : "");
-	printf ("%ld block%s\n", ZONES, (ZONES != 1) ? "s" : "");
-	printf ("%d bad block%s\n", badblocks, (badblocks != 1) ? "s" : "");
-	printf ("Firstdatazone=%ld (%d)\n", FIRSTZONE, NORM_FIRSTZONE);
-	printf ("%ld free block%s\n", FREEBLOCKSCOUNT,
+	printf ("%lu inode%s\n", (unsigned long) INODES, 
+		(INODES != 1) ? "s" : "");
+	printf ("%lu block%s\n", (unsigned long) ZONES, 
+		(ZONES != 1) ? "s" : "");
+	printf ("%u bad block%s\n", (unsigned int) badblocks, 
+		(badblocks != 1) ? "s" : "");
+	printf ("Firstdatazone=%lu (%u)\n", (unsigned long) FIRSTZONE, 
+		(unsigned int) NORM_FIRSTZONE);
+	printf ("%lu free block%s\n", (unsigned long) FREEBLOCKSCOUNT,
 		(FREEBLOCKSCOUNT != 1) ? "s" : "");
-	printf ("%ld free inode%s\n", FREEINODESCOUNT,
+	printf ("%lu free inode%s\n", (unsigned long) FREEINODESCOUNT,
 		(FREEINODESCOUNT != 1) ? "s" : "");
-	printf ("Zonesize=%d\n", BLOCK_SIZE << ZONESIZE);
+	printf ("Zonesize=%u\n", (unsigned int) (BLOCK_SIZE << ZONESIZE));
     }
 }
 
@@ -184,7 +204,7 @@ void init_inode_bitmap (void)
 	inode_map = malloc (size * groups);
 	if (!inode_map)
 		die ("Unable to allocate inodes bitmap\n");
-	memset (inode_map, 0, (INODES / 8) + 1);
+	memset (inode_map, 0, ((INODES + 1) / 8));
 
         for (i = 0; i < groups; i++) {
                 pos = bg[i].bg_inode_bitmap * block_size;
@@ -259,17 +279,18 @@ static void count_free_blocks(void) {
                count++;
        gp[n].free_blocks = count;
        total_free += count;
-       if (debug) printf("Group %d, free blocks %lu\n",n,count);        
-       if (count!=bg[n].bg_free_blocks_count)
-          fprintf(stderr,"Group %d: free_blocks:%u counted:%lu\n",n,
-                  bg[n].bg_free_blocks_count,count);
+       if (debug) printf("Group %d, free blocks %lu\n", n, count);        
+       if (count != bg[n].bg_free_blocks_count)
+          fprintf(stderr, "Group %d: free_blocks:%u counted:%lu\n", n,
+                  bg[n].bg_free_blocks_count, count);
    }
-   if (debug) printf("Total free blocks counted: %lu\n",total_free);
-   if (FREEBLOCKSCOUNT!=total_free) {
+   if (debug) printf("Total free blocks counted: %lu\n", total_free);
+   if (FREEBLOCKSCOUNT != total_free) {
        char s[256];
-       sprintf(s,"Free blocks count wrong, free:"
-	       "%lu, reserved:%lu, counted:%lu\nRun fsck.\n",
-              FREEBLOCKSCOUNT,Super.s_r_blocks_count,total_free);
+       sprintf (s, "Free blocks count wrong, free:"
+		"%lu, reserved:%lu, counted:%lu\nRun fsck.\n",
+		(unsigned long) FREEBLOCKSCOUNT, 
+		(unsigned long) Super.s_r_blocks_count, total_free);
        fatal_error(s);
    }       
 }                                  
